
                    ########################################
                    ########################################
                    ##                                    ##
                    ##      Cours A2, formation ISN       ##
                    ##                                    ##
                    ##              Julien Lefevre        ##
                    ########################################
                    ########################################
 
import numpy
import scipy
import scipy.fftpack
import scipy.ndimage.filters as filt
import copy
from math import *
import pylab as pl

###########################
###########################
# 1) Echantillonnage      #
###########################

###########################
# Images avec matplotlib  #
###########################

def VisuImage(tab): # Visualisateur simple en niveau de gris et image redressee
    pl.figure()
    pl.imshow(tab,cmap=pl.cm.gray,origin='lower')
    pl.axis("off")
    

############################
# Sous-echantillonage
############################

def sous_echantillonnage(Im,ne,scale=False):
    Im2=numpy.zeros((Im.shape[0]/ne,Im.shape[1]/ne),dtype='float')
    for x in range(Im2.shape[0]):
        for y in range(Im2.shape[1]):
            Im2[x][y]=Im[ne*x][ne*y]
    VisuImage(Im2)
    pl.axis("off")
    if scale:
        pl.xlim((0,Im.shape[0]))
        pl.xlim((0,Im.shape[1]))     


###########################
###########################
# 2) Conversion           #
###########################


############################
# Quantification           #
############################

    
def find_nearest(data,val): # trouve la valeur la plus proche de val dans le tableau data
    idx=(numpy.abs(data-val)).argmin()
    return data[idx]

def quantification(data,Nvaleur): # quantification sur Nvaleur
    M=data.max()
    m=data.min()
    quant=numpy.linspace(m,M,Nvaleur)
    data_quant=[find_nearest(quant,s) for s in data]
    return data_quant


# Quantification pour image

def visu_quantification(tab,Nvaleur):
    (m,n)=tab.shape
    tab=tab.reshape(m*n,)
    tab_quant=quantification(tab,Nvaleur)
    tab_quant=(numpy.array(tab_quant)).reshape(m,n)
    VisuImage(tab_quant)
    

############################
############################
# 3) Operations numeriques #
############################


############################
# Histo
############################

def histograme2(data,VERBOSE): # histogramme d'un tableau de valeurs numeriques
    dico={} # dico est un dictionnaire dont les clefs vont varier entre min(data) et max(data)
    for i in range(data.shape[0]):
        for j in range(data.shape[1]):
            x=data[i][j]
            if x in dico:
                dico[x]=dico[x]+1
            else:
                dico[x]=1
    if VERBOSE:
        pl.plot(dico.keys(),dico.values(),'+')
        pl.xlabel('Niveaux de gris')
        pl.ylabel('Nombre de pixels')
    return dico
    
############################
# Seuillage
############################

def seuillage(tab,seuil): # seuillage binaire d'une image
    tab2=copy.deepcopy(tab)
    for i in range(tab.shape[0]):
        for j in range(tab.shape[1]):
            if tab[i][j]>seuil:
                tab2[i][j]=255
            else:
                tab2[i][j]=0    
    return tab2        

############################
# Contraste                #
############################


def contraste(tab,Imin,Imax): # changement de contraste lineaire 
    M=tab.max()
    m=tab.min()
    transfo=numpy.linspace(m,M,M-m+1)
    tab2=numpy.zeros(tab.shape,dtype='float')
    for x in range(len(transfo)):
        transfo[x]=(x+m)*(Imax-Imin)/(M-m)+(Imin*M-Imax*m)/(M-m)
    for x in range(tab.shape[0]):
        for y in range(tab.shape[1]):
            tab2[x][y]=transfo[tab[x][y]-m]
    return tab2


def VisuImage2(tab): # Visualisateur ou on ne represente que les niveaux de gris entre 0 et 255
    pl.figure()
    pl.imshow(tab,cmap=pl.cm.gray,vmin=0,vmax=255,origin='lower')
    pl.axis("off")
    
############################
# Filtrage sans scipy      #
############################

def Mirroir(tab,i,j): # on symetrise par rapport aux bords
    m=tab.shape[0]
    n=tab.shape[1]
    if (i<0):
        i=-i-1
    elif (i>m-1):
        i=2*m-i-1
    if (j<0):
        j=-j-1
    elif (j>n-1):
        j=2*n-j-1
    return tab[i,j]
        

def filtrage(tab,filtre):  # equivalent de convolve dans scipy.ndimage.filters
    tab_filtre=copy.deepcopy(tab)
    m=filtre.shape[0]
    n=filtre.shape[1]
    for i in range(tab.shape[0]):
        for j in range(tab.shape[1]):
            somme=0.0
            for k in range(-m/2,m/2+1,1):
                for l in range(-n/2,n/2+1,1):
                    somme=somme+Mirroir(tab,i+k,j+l)*filtre[k+m/2,l+n/2]
            tab_filtre[i,j]=somme
    return tab_filtre
